//--------------------------------------------------------------
// File     : stm32_ub_lin_slave.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_LIN_SLAVE_H
#define __STM32F4_UB_LIN_SLAVE_H



//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "misc.h"



//--------------------------------------------------------------
// GPIO : UART-TX (PA2)
//--------------------------------------------------------------
#define  LIN_TX_CLOCK    RCC_AHB1Periph_GPIOA
#define  LIN_TX_PIN      GPIO_Pin_2
#define  LIN_TX_SOURCE   GPIO_PinSource2
#define  LIN_TX_PORT     GPIOA


//--------------------------------------------------------------
// GPIO : UART-RX (PA3)
//--------------------------------------------------------------
#define  LIN_RX_CLOCK    RCC_AHB1Periph_GPIOA
#define  LIN_RX_PIN      GPIO_Pin_3
#define  LIN_RX_SOURCE   GPIO_PinSource3
#define  LIN_RX_PORT     GPIOA


//--------------------------------------------------------------
// UART : (UART-2)
//--------------------------------------------------------------
#define  LIN_UART        USART2
#define  LIN_UART_CLOCK  RCC_APB1Periph_USART2
#define  LIN_UART_AF     GPIO_AF_USART2
#define  LIN_UART_IRQ    USART2_IRQn
#define  LIN_UART_ISR    USART2_IRQHandler


//--------------------------------------------------------------
// GPIO : NSLP-Pin vom Transceiver (PA1)
//--------------------------------------------------------------
#define  LIN_NSLP_CLOCK    RCC_AHB1Periph_GPIOA
#define  LIN_NSLP_PIN      GPIO_Pin_1
#define  LIN_NSLP_PORT     GPIOA



//--------------------------------------------------------------
// Baudrate (max 20 kBaud)
//
// @9600 Baud : Fieldzeit (10bit) = ca. 1ms
//--------------------------------------------------------------
#define  LIN_UART_BAUD   9600



//--------------------------------------------------------------
// Globale Defines
//--------------------------------------------------------------
#define  LIN_SYNC_DATA               0x55  // SyncField (nicht aendern)
#define  LIN_MAX_DATA                   8  // max 8 Datenytes



//--------------------------------------------------------------
// Defines fuer den Transceiver (nicht aendern !!)
//--------------------------------------------------------------
#define  LIN_POWERON_DELAY          10000  // Pause beim PowerOn     (ca. 10ms)
#define  LIN_AKTIV_DELAY               50  // Pause fuer Transceiver (ca. 50us)



//--------------------------------------------------------------
// Pausenzeiten (Vorsicht beim aendern !!)
//--------------------------------------------------------------
#define  LIN_INTER_FRAME_DELAY      10000  // Pause (Frame->Frame)   (ca. 10ms)
#define  LIN_FRAME_RESPONSE_DELAY    2000  // Pause (Header->Data)   (ca.  2ms)
#define  LIN_BREAKFIELD_DELAY        4000  // Pause (Breakfield)     (ca.  4ms)
#define  LIN_DATA_BYTE_DELAY         1000  // Pause (Data->Data)     (ca.  1ms)
#define  LIN_RX_TIMEOUT_CNT         30000  // timeout beim empfangen (ca.  5ms)



//--------------------------------------------------------------
// Struktur von einem LIN-Frame
//--------------------------------------------------------------
typedef struct {
  uint8_t frame_id;              // eindeutige ID-Nummer vom Frame
  uint8_t data_len;              // Anzahl der Daten
  uint8_t data[LIN_MAX_DATA];    // Datenfelder
}LIN_FRAME_t;



//--------------------------------------------------------------
// Mode vom Slave
//--------------------------------------------------------------
typedef enum { 
  WAIT_4_BREAK =0,  // warte auf BreakField
  RECEIVE_SYNC,     // SYNC empfangen
  RECEIVE_ID,       // ID empfangen
  ID_OK,            // ID ok
  WAIT_4_CMD,       // warte auf Kommando
  RECEIVE_DATA,     // Daten empfangen
  RECEIVE_CRC,      // CRC empfangen
  SEND_DATA         // Daten senden
}LIN_MODE_t;



//--------------------------------------------------------------
// Struktur vom Slave
//--------------------------------------------------------------
typedef struct {
  LIN_MODE_t mode;   // aktueller Mode
  uint8_t data_ptr;  // Datenpointer  
  uint8_t crc;       // Checksumme
}LIN_SLAVE_t;



//--------------------------------------------------------------
// Fehlermeldungen
//--------------------------------------------------------------
typedef enum {
  LIN_OK  = 0,   // kein Fehler
  LIN_WRONG_LEN, // falsche Datenanzahl
  LIN_RX_EMPTY,  // kein Frame empfangen
  LIN_WRONG_CRC, // Checksumme falsch
  LIN_NO_ID      // es wurde keine ID empfangen
}LIN_ERR_t;




//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_LIN_Slave_Init(void);
LIN_ERR_t UB_LIN_checkID(LIN_FRAME_t *frame);
LIN_ERR_t UB_LIN_SendData(LIN_FRAME_t *frame);
LIN_ERR_t UB_LIN_ReceiveData(LIN_FRAME_t *frame);




//--------------------------------------------------------------
#endif // __STM32F4_UB_LIN_SLAVE_H
